@@ -36,6 +36,8 @@ _vte_ring_validate(VteRing * ring)
 		g_assert(ring->array[i % ring->max] != NULL);
 	}
 }
+#else
+#define _vte_ring_validate(ring)
 #endif
 
 /**
@@ -55,7 +57,7 @@ _vte_ring_new(glong max_elements, VteRingFreeFunc free, gpointer data)
 {
 	VteRing *ret = g_slice_new0(VteRing);
 	ret->user_data = data;
-	ret->delta = ret->length = 0;
+	ret->cached_item = -1;
 	ret->max = MAX(max_elements, 2);
 	ret->array = g_malloc0(sizeof(gpointer) * ret->max);
 	ret->free = free;
@@ -83,56 +85,51 @@ _vte_ring_new_with_delta(glong max_elements, glong delta,
  * being replaced by the new @data.
  *
  */
-void
+gpointer
 _vte_ring_insert(VteRing * ring, long position, gpointer data)
 {
+	gpointer old_data = NULL;
 	long point, i;
-#ifdef VTE_DEBUG
-	if (_vte_debug_on(VTE_DEBUG_RING)) {
-		g_printerr("Inserting at position %ld.\n", position);
-		g_printerr(" Delta = %ld, Length = %ld, Max = %ld.\n",
-			ring->delta, ring->length, ring->max);
-	}
+
+	g_return_val_if_fail(ring != NULL, NULL);
+	g_return_val_if_fail(position >= ring->delta, NULL);
+	g_return_val_if_fail(position <= ring->delta + ring->length, NULL);
+	g_return_val_if_fail(data != NULL, NULL);
+
+	_vte_debug_print(VTE_DEBUG_RING,
+			"Inserting at position %ld.\n"
+			" Delta = %ld, Length = %ld, Max = %ld.\n",
+			position, ring->delta, ring->length, ring->max);
 	_vte_ring_validate(ring);
-#endif
-	g_return_if_fail(ring != NULL);
-	g_return_if_fail(position >= ring->delta);
-	g_return_if_fail(position <= ring->delta + ring->length);
-	g_return_if_fail(data != NULL);
 
 	/* Initial insertion, or append. */
 	if (position == ring->length + ring->delta) {
-		/* If there was something there before, free it. */
-		if ((ring->free != NULL) &&
-		    (ring->array[position % ring->max] != NULL)) {
-#ifdef VTE_DEBUG
-			if (_vte_debug_on(VTE_DEBUG_RING)) {
-				g_printerr("Freeing item at position "
-					"%ld.\n", position);
-			}
-#endif
-			ring->free(ring->array[position % ring->max],
-				   ring->user_data);
-		}
+		i = position % ring->max;
+		old_data = ring->array[i];
 		/* Set the new item, and if the buffer wasn't "full", increase
 		 * our idea of how big it is, otherwise increase the delta so
 		 * that this becomes the "last" item and the previous item
 		 * scrolls off the *top*. */
-		ring->array[position % ring->max] = data;
+		ring->array[i] = data;
 		if (ring->length == ring->max) {
 			ring->delta++;
+			if (ring->delta > ring->cached_item) {
+				_vte_ring_set_cache (ring, -1, NULL);
+			}
+
 		} else {
 			ring->length++;
 		}
-#ifdef VTE_DEBUG
-		if (_vte_debug_on(VTE_DEBUG_RING)) {
-			g_printerr(" Delta = %ld, Length = %ld, "
+		_vte_debug_print(VTE_DEBUG_RING,
+				" Delta = %ld, Length = %ld, "
 				"Max = %ld.\n",
 				ring->delta, ring->length, ring->max);
-		}
 		_vte_ring_validate(ring);
-#endif
-		return;
+		return old_data;
+	}
+
+	if (position <= ring->cached_item) {
+		_vte_ring_set_cache (ring, -1, NULL);
 	}
 
 	/* All other cases.  Calculate the location where the last "item" in the
@@ -146,16 +143,7 @@ _vte_ring_insert(VteRing * ring, long position, gpointer data)
 		/* If the buffer's full, then the last item will have to be
 		 * "lost" to make room for the new item so that the buffer
 		 * doesn't grow (here we scroll off the *bottom*). */
-		if (ring->free && ring->array[point % ring->max]) {
-#ifdef VTE_DEBUG
-			if (_vte_debug_on(VTE_DEBUG_RING)) {
-				g_printerr("Freeing item at position "
-					"%ld.\n", point);
-			}
-#endif
-			ring->free(ring->array[point % ring->max],
-				   ring->user_data);
-		}
+		old_data = ring->array[point % ring->max];
 	} else {
 		/* We don't want to discard the last item. */
 		point++;
@@ -171,13 +159,12 @@ _vte_ring_insert(VteRing * ring, long position, gpointer data)
 	 * maximum length already. */
 	ring->array[position % ring->max] = data;
 	ring->length = CLAMP(ring->length + 1, 0, ring->max);
-#ifdef VTE_DEBUG
-	if (_vte_debug_on(VTE_DEBUG_RING)) {
-		g_printerr(" Delta = %ld, Length = %ld, Max = %ld.\n",
+	_vte_debug_print(VTE_DEBUG_RING,
+			" Delta = %ld, Length = %ld, Max = %ld.\n",
 			ring->delta, ring->length, ring->max);
-	}
 	_vte_ring_validate(ring);
-#endif
+
+	return old_data;
 }
 
 /**
@@ -192,29 +179,30 @@ _vte_ring_insert(VteRing * ring, long position, gpointer data)
  * from the *top*.
  *
  */
-void
+gpointer
 _vte_ring_insert_preserve(VteRing * ring, long position, gpointer data)
 {
 	long point, i;
-	gpointer **tmp;
+	gpointer **tmp, old_data = NULL;
 
-	g_return_if_fail(position <= _vte_ring_next(ring));
+	g_return_val_if_fail(position <= _vte_ring_next(ring), NULL);
 
-#ifdef VTE_DEBUG
-	if (_vte_debug_on(VTE_DEBUG_RING)) {
-		g_printerr("Inserting+ at position %ld.\n", position);
-		g_printerr(" Delta = %ld, Length = %ld, Max = %ld.\n",
-			ring->delta, ring->length, ring->max);
-	}
+	_vte_debug_print(VTE_DEBUG_RING,
+			"Inserting+ at position %ld.\n"
+			" Delta = %ld, Length = %ld, Max = %ld.\n",
+			position, ring->delta, ring->length, ring->max);
 	_vte_ring_validate(ring);
-#endif
+
+	if (position <= ring->cached_item) {
+		_vte_ring_set_cache (ring, -1, NULL);
+	}
 
 	/* Allocate space to save existing elements. */
 	point = _vte_ring_next(ring);
 	i = MAX(1, point - position);
 
 	/* Save existing elements. */
-	tmp = g_malloc0(sizeof(gpointer) * i);
+	tmp = g_new0(gpointer *, i);
 	for (i = position; i < point; i++) {
 		tmp[i - position] = _vte_ring_index(ring, gpointer, i);
 	}
@@ -225,15 +213,20 @@ _vte_ring_insert_preserve(VteRing * ring, long position, gpointer data)
 	}
 
 	/* Append the new item. */
-	_vte_ring_append(ring, data);
+	old_data = _vte_ring_append(ring, data);
 
 	/* Append the old items. */
 	for (i = position; i < point; i++) {
-		_vte_ring_append(ring, tmp[i - position]);
+		if (old_data && ring->free) {
+			ring->free (old_data, ring->user_data);
+		}
+		old_data = _vte_ring_append(ring, tmp[i - position]);
 	}
 
 	/* Clean up. */
 	g_free(tmp);
+
+	return old_data;
 }
 
 /**
@@ -246,29 +239,31 @@ _vte_ring_insert_preserve(VteRing * ring, long position, gpointer data)
  * %TRUE.
  *
  */
-void
+gpointer
 _vte_ring_remove(VteRing * ring, long position, gboolean free)
 {
 	long i;
-#ifdef VTE_DEBUG
-	if (_vte_debug_on(VTE_DEBUG_RING)) {
-		g_printerr("Removing item at position %ld.\n", position);
-		g_printerr(" Delta = %ld, Length = %ld, Max = %ld.\n",
-			ring->delta, ring->length, ring->max);
-	}
+	gpointer old_data;
+	_vte_debug_print(VTE_DEBUG_RING,
+			"Removing item at position %ld.\n"
+			" Delta = %ld, Length = %ld, Max = %ld.\n",
+			position, ring->delta, ring->length, ring->max);
 	_vte_ring_validate(ring);
-#endif
-	/* Remove the data at this position. */
-	if (free && ring->array[position % ring->max] && ring->free) {
-#ifdef VTE_DEBUG
-		if (_vte_debug_on(VTE_DEBUG_RING)) {
-			g_printerr("Freeing item at position %ld.\n",
-				position);
-		}
-#endif
-		ring->free(ring->array[position % ring->max], ring->user_data);
+
+	if (position <= ring->cached_item) {
+		_vte_ring_set_cache (ring, -1, NULL);
 	}
-	ring->array[position % ring->max] = NULL;
+
+	i = position % ring->max;
+	/* Remove the data at this position. */
+	old_data = ring->array[i];
+	if (free && old_data && ring->free) {
+		_vte_debug_print(VTE_DEBUG_RING,
+				"Freeing item at position %ld.\n", position);
+		ring->free(old_data, ring->user_data);
+		old_data = NULL;
+	}
+	ring->array[i] = NULL;
 
 	/* Bubble the rest of the buffer up one notch.  This is also less
 	 * of a problem than it might appear, again due to usage patterns. */
@@ -282,13 +277,12 @@ _vte_ring_remove(VteRing * ring, long position, gboolean free)
 	if (ring->length > 0) {
 		ring->length--;
 	}
-#ifdef VTE_DEBUG
-	if (_vte_debug_on(VTE_DEBUG_RING)) {
-		g_printerr(" Delta = %ld, Length = %ld, Max = %ld.\n",
+	_vte_debug_print(VTE_DEBUG_RING,
+			" Delta = %ld, Length = %ld, Max = %ld.\n",
 			ring->delta, ring->length, ring->max);
-	}
 	_vte_ring_validate(ring);
-#endif
+
+	return old_data;
 }
 
 /**
@@ -300,11 +294,11 @@ _vte_ring_remove(VteRing * ring, long position, gboolean free)
  * the new item, it is freed.
  *
  */
-void
+gpointer
 _vte_ring_append(VteRing * ring, gpointer data)
 {
 	g_assert(data != NULL);
-	_vte_ring_insert(ring, ring->delta + ring->length, data);
+	return _vte_ring_insert(ring, ring->delta + ring->length, data);
 }
 
 /**
