@@ -639,13 +639,13 @@ _vte_table_match(struct _vte_table *table,
 
 	/* If there's no literal path, and no generic path, and the numeric
 	 * path isn't available, then it's not a sequence, either. */
-	if ((table->table == NULL) ||
-	    (table->table[_vte_table_map_literal(candidate[0])] == NULL)) {
+	if (table->table == NULL ||
+	    table->table[_vte_table_map_literal(candidate[0])] == NULL) {
 		if (table->table_string == NULL) {
-			if (!(_vte_table_is_numeric(candidate[0])) ||
-			    (table->table_number == NULL)) {
-				if (!(_vte_table_is_numeric_list(candidate[0])) ||
-						(table->table_number_list == NULL)) {
+			if (table->table_number == NULL ||
+					!_vte_table_is_numeric(candidate[0])){
+				if (table->table_number_list == NULL ||
+					!_vte_table_is_numeric_list(candidate[0])){
 					/* No match. */
 					return NULL;
 				}
@@ -683,7 +683,7 @@ _vte_table_match(struct _vte_table *table,
 		g_assert(original != NULL);
 		p = original;
 		arginfo = _vte_table_arginfo_head_reverse (&params);
-		while (p < original + original_length) {
+		do {
 			/* All of the interesting arguments begin with '%'. */
 			if (p[0] == '%') {
 				/* Handle an increment. */
@@ -718,12 +718,12 @@ _vte_table_match(struct _vte_table *table,
 								arginfo,
 								p[2]);
 					p += 2;
+				} else {
+					g_assert_not_reached();
 				}
-				g_assert_not_reached();
 			} /* else Literal. */
 			arginfo = arginfo->next;
-			p++;
-		}
+		} while (++p < original + original_length && arginfo);
 	}
 
 	/* Clean up extracted parameters. */
