@@ -74,8 +74,8 @@ struct char_class {
 					   counted together. */
 	gunichar *code;			/* A magic string that indicates this
 					   class should be found here. */
-	size_t code_length;
-	size_t ccount;			/* The maximum number of characters
+	gsize code_length;
+	gsize ccount;			/* The maximum number of characters
 					   after the format specifier to
 					   consume. */
 	gboolean (*check)(const gunichar c, struct char_class_data *data);
@@ -84,7 +84,7 @@ struct char_class {
 	void (*setup)(const gunichar *s, struct char_class_data *data, int inc);
 					/* Setup the data struct for use in the
 					 * above check function. */
-	gboolean (*extract)(const gunichar *s, size_t length,
+	gboolean (*extract)(const gunichar *s, gsize length,
 			    struct char_class_data *data,
 			    GValueArray *array);
 					/* Extract a parameter. */
@@ -92,11 +92,12 @@ struct char_class {
 
 /* A trie to hold control sequences. */
 struct _vte_trie {
+	struct _vte_matcher_impl impl;
 	const char *result;		/* If this is a terminal node, then this
 					   field contains its "value". */
 	GQuark quark;			/* The quark for the value of the
 					   result. */
-	size_t trie_path_count;		/* Number of children of this node. */
+	gsize trie_path_count;		/* Number of children of this node. */
 	struct trie_path {
 		struct char_class *cclass;
 		struct char_class_data data;
@@ -126,7 +127,7 @@ char_class_percent_setup(const gunichar *s, struct char_class_data *data,
 	return;
 }
 static gboolean
-char_class_none_extract(const gunichar *s, size_t length,
+char_class_none_extract(const gunichar *s, gsize length,
 			struct char_class_data *data, GValueArray *array)
 {
 	return FALSE;
@@ -158,11 +159,11 @@ char_class_digit_setup(const gunichar *s, struct char_class_data *data, int inc)
 	return;
 }
 static gboolean
-char_class_digit_extract(const gunichar *s, size_t length,
+char_class_digit_extract(const gunichar *s, gsize length,
 			 struct char_class_data *data, GValueArray *array)
 {
 	long ret = 0;
-	size_t i;
+	gsize i;
 	GValue value;
 	for (i = 0; i < length; i++) {
 		ret *= 10;
@@ -204,11 +205,11 @@ char_class_multi_setup(const gunichar *s, struct char_class_data *data, int inc)
 	return;
 }
 static gboolean
-char_class_multi_extract(const gunichar *s, size_t length,
+char_class_multi_extract(const gunichar *s, gsize length,
 			 struct char_class_data *data, GValueArray *array)
 {
 	long ret = 0;
-	size_t i;
+	gsize i;
 	GValue value;
 	memset(&value, 0, sizeof(value));
 	g_value_init(&value, G_TYPE_LONG);
@@ -240,7 +241,7 @@ char_class_any_setup(const gunichar *s, struct char_class_data *data, int inc)
 	return;
 }
 static gboolean
-char_class_any_extract(const gunichar *s, size_t length,
+char_class_any_extract(const gunichar *s, gsize length,
 		       struct char_class_data *data, GValueArray *array)
 {
 	long ret = 0;
@@ -265,10 +266,10 @@ char_class_string_setup(const gunichar *s, struct char_class_data *data, int inc
 	data->c = s[0];
 	return;
 }
-static size_t
-unichar_snlen(const gunichar *s, size_t length)
+static gsize
+unichar_snlen(const gunichar *s, gsize length)
 {
-	size_t i;
+	gsize i;
 	for (i = 0; i < length; i++) {
 		if (s[i] == '\0') {
 			return i;
@@ -288,9 +289,9 @@ unichar_sncpy(gunichar *d, const gunichar *s, gsize length)
 	}
 }
 static int
-unichar_sncmp(const gunichar *a, const gunichar *b, size_t length)
+unichar_sncmp(const gunichar *a, const gunichar *b, gsize length)
 {
-	int i;
+	gsize i;
 	for (i = 0; i < length; i++) {
 		if (a[i] != b[i]) {
 			return a[i] - b[i];
@@ -302,12 +303,12 @@ unichar_sncmp(const gunichar *a, const gunichar *b, size_t length)
 	return 0;
 }
 static gboolean
-char_class_string_extract(const gunichar *s, size_t length,
+char_class_string_extract(const gunichar *s, gsize length,
 			  struct char_class_data *data, GValueArray *array)
 {
 	gunichar *ret = NULL;
-	size_t len;
-	int i;
+	gsize len;
+	gsize i;
 	GValue value;
 
 	len = unichar_snlen(s, length);
@@ -316,11 +317,8 @@ char_class_string_extract(const gunichar *s, size_t length,
 	for (i = 0; i < len; i++) {
 		ret[i] &= ~(VTE_ISO2022_ENCODED_WIDTH_MASK);
 	}
-#ifdef VTE_DEBUG
-	if (_vte_debug_on(VTE_DEBUG_PARSE)) {
-		fprintf(stderr, "Extracting string `%ls'.\n", (wchar_t*) ret);
-	}
-#endif
+	_vte_debug_print(VTE_DEBUG_PARSE,
+			"Extracting string `%ls'.\n", (wchar_t*) ret);
 	memset(&value, 0, sizeof(value));
 
 	g_value_init(&value, G_TYPE_POINTER);
@@ -374,7 +372,10 @@ static struct char_class char_classes[] = {
 TRIE_MAYBE_STATIC struct _vte_trie *
 _vte_trie_new(void)
 {
-	return g_slice_new0(struct _vte_trie);
+	struct _vte_trie *ret;
+	ret = g_slice_new0(struct _vte_trie);
+	ret->impl.klass = &_vte_matcher_trie;
+	return ret;
 }
 
 TRIE_MAYBE_STATIC void
@@ -393,14 +394,14 @@ _vte_trie_free(struct _vte_trie *trie)
 /* Add the given pattern, with its own result string, to the trie, with the
  * given initial increment value. */
 static void
-_vte_trie_addx(struct _vte_trie *trie, gunichar *pattern, size_t length,
+_vte_trie_addx(struct _vte_trie *trie, gunichar *pattern, gsize length,
 	       const char *result, GQuark quark, int inc)
 {
-	unsigned long i;
+	gsize i;
 	struct char_class *cclass = NULL;
 	struct char_class_data data;
 	gunichar *code;
-	size_t len = 0, ccount = 0;
+	gsize len = 0, ccount = 0;
 	gunichar inc_wstring[] = {'%', 'i', '\0'};
 
 	/* The trivial case -- we'll just set the result at this node. */
@@ -409,12 +410,9 @@ _vte_trie_addx(struct _vte_trie *trie, gunichar *pattern, size_t length,
 			trie->quark = g_quark_from_string(result);
 			trie->result = g_quark_to_string(trie->quark);
 		} else {
-#ifdef VTE_DEBUG
-			if (_vte_debug_on(VTE_DEBUG_PARSE)) {
+			_VTE_DEBUG_IF(VTE_DEBUG_PARSE)
 				g_warning(_("Duplicate (%s/%s)!"),
 					  result, trie->result);
-			}
-#endif
 		}
 		return;
 	}
@@ -430,7 +428,7 @@ _vte_trie_addx(struct _vte_trie *trie, gunichar *pattern, size_t length,
 
 	/* Now check for examples of character class specifiers, and use that
 	 * to put this part of the pattern in a character class. */
-	for (i = G_N_ELEMENTS(char_classes) - 1; i >= 0; i--) {
+	for (i = G_N_ELEMENTS(char_classes); i--; ) {
 		len = char_classes[i].code_length;
 		code = char_classes[i].code;
 		ccount = char_classes[i].ccount;
@@ -439,7 +437,6 @@ _vte_trie_addx(struct _vte_trie *trie, gunichar *pattern, size_t length,
 			break;
 		}
 	}
-	g_assert(i >= 0);
 
 	/* Initialize the data item using the data we have here. */
 	memset(&data, 0, sizeof(data));
@@ -485,12 +482,13 @@ _vte_trie_addx(struct _vte_trie *trie, gunichar *pattern, size_t length,
 
 /* Add the given pattern, with its own result string, to the trie. */
 TRIE_MAYBE_STATIC void
-_vte_trie_add(struct _vte_trie *trie, const char *pattern, size_t length,
+_vte_trie_add(struct _vte_trie *trie, const char *pattern, gsize length,
 	      const char *result, GQuark quark)
 {
-	char *wpattern, *wpattern_end, *tpattern;
+	const guchar *tpattern;
+	guchar *wpattern, *wpattern_end;
 	VteConv conv;
-	size_t wlength;
+	gsize wlength;
 
 	g_return_if_fail(trie != NULL);
 	g_return_if_fail(pattern != NULL);
@@ -504,9 +502,9 @@ _vte_trie_add(struct _vte_trie *trie, const char *pattern, size_t length,
 	wpattern = wpattern_end = g_malloc0(wlength + 1);
 
 	conv = _vte_conv_open(VTE_CONV_GUNICHAR_TYPE, "UTF-8");
-	g_assert(conv != ((VteConv) -1));
+	g_assert(conv != VTE_INVALID_CONV);
 
-	tpattern = (char*)pattern;
+	tpattern = (const guchar *)pattern;
 	_vte_conv(conv, &tpattern, &length, &wpattern_end, &wlength);
 	if (length == 0) {
 		wlength = (wpattern_end - wpattern) / sizeof(gunichar);
@@ -519,10 +517,10 @@ _vte_trie_add(struct _vte_trie *trie, const char *pattern, size_t length,
 }
 
 /* Check if the given pattern matches part of the given trie, returning an
- * empty string on a partial initial match, a NULL if there's no match in the
+ * empty string on a partial initial match, a %NULL if there's no match in the
  * works, and the result string if we have an exact match. */
 static const char *
-_vte_trie_matchx(struct _vte_trie *trie, const gunichar *pattern, size_t length,
+_vte_trie_matchx(struct _vte_trie *trie, const gunichar *pattern, gsize length,
 		 gboolean greedy,
 		 const char **res, const gunichar **consumed,
 		 GQuark *quark, GValueArray *array)
@@ -634,13 +632,15 @@ _vte_trie_matchx(struct _vte_trie *trie, const gunichar *pattern, size_t length,
 				if (better) {
 					best = tmp;
 					if (bestarray != NULL) {
-						_vte_matcher_free_params_array(bestarray);
+						_vte_matcher_free_params_array(
+								NULL, bestarray);
 					}
 					bestarray = tmparray;
 					bestquark = tmpquark;
 					bestconsumed = *consumed;
 				} else {
-					_vte_matcher_free_params_array(tmparray);
+					_vte_matcher_free_params_array(
+							NULL, tmparray);
 					tmparray = NULL;
 				}
 			}
@@ -652,12 +652,12 @@ _vte_trie_matchx(struct _vte_trie *trie, const gunichar *pattern, size_t length,
 		for (i = 0; i < bestarray->n_values; i++) {
 			GValue *value = g_value_array_get_nth(bestarray, i);
 			g_value_array_append(array, value);
-			
+
 			if (G_VALUE_HOLDS_POINTER(value)) {
 				g_value_set_pointer(value, NULL);
 			}
 		}
-		_vte_matcher_free_params_array(bestarray);
+		_vte_matcher_free_params_array(NULL, bestarray);
 	}
 #if 0
 	printf("`%s' ", best);
@@ -670,10 +670,10 @@ _vte_trie_matchx(struct _vte_trie *trie, const gunichar *pattern, size_t length,
 }
 
 /* Check if the given pattern matches part of the given trie, returning an
- * empty string on a partial initial match, a NULL if there's no match in the
+ * empty string on a partial initial match, a %NULL if there's no match in the
  * works, and the result string if we have an exact match. */
 TRIE_MAYBE_STATIC const char *
-_vte_trie_match(struct _vte_trie *trie, const gunichar *pattern, size_t length,
+_vte_trie_match(struct _vte_trie *trie, const gunichar *pattern, gsize length,
 		const char **res, const gunichar **consumed,
 		GQuark *quark, GValueArray **array)
 {
@@ -682,11 +682,14 @@ _vte_trie_match(struct _vte_trie *trie, const gunichar *pattern, size_t length,
 	GValueArray *valuearray;
 	GValue *value;
 	const gunichar *dummyconsumed;
-	gpointer ptr;
 	gboolean greedy = FALSE;
-	int i;
+	guint i;
 
-	valuearray = g_value_array_new(0);
+	if (array != NULL && *array != NULL) {
+		valuearray = *array;
+	} else {
+		valuearray = g_value_array_new(0);
+	}
 	if (quark == NULL) {
 		quark = &tmpquark;
 	}
@@ -705,23 +708,17 @@ _vte_trie_match(struct _vte_trie *trie, const gunichar *pattern, size_t length,
 			for (i = 0; i < valuearray->n_values; i++) {
 				value = g_value_array_get_nth(valuearray, i);
 				if (G_VALUE_HOLDS_POINTER(value)) {
-					ptr = g_value_get_pointer(value);
-					if (ptr != NULL) {
-						g_free(ptr);
-					}
+					g_free(g_value_get_pointer(value));
 					g_value_set_pointer(value, NULL);
 				}
 			}
-			_vte_matcher_free_params_array(valuearray);
-		}
-		if (array != NULL) {
-			*array = NULL;
+			if (array == NULL || valuearray != *array) {
+				_vte_matcher_free_params_array(NULL, valuearray);
+			}
 		}
 	} else {
-		if (array != NULL) {
-			*array = valuearray;
-		} else {
-			_vte_matcher_free_params_array(valuearray);
+		if (array == NULL) {
+			_vte_matcher_free_params_array(NULL, valuearray);
 		}
 	}
 
@@ -731,7 +728,7 @@ _vte_trie_match(struct _vte_trie *trie, const gunichar *pattern, size_t length,
 /* Print the next layer of the trie, indented by length spaces. */
 static void
 _vte_trie_printx(struct _vte_trie *trie, const char *previous,
-		 size_t *nodecount)
+		 gsize *nodecount)
 {
 	unsigned int i;
 	char buf[LINE_MAX];
@@ -809,7 +806,7 @@ _vte_trie_printx(struct _vte_trie *trie, const char *previous,
 TRIE_MAYBE_STATIC void
 _vte_trie_print(struct _vte_trie *trie)
 {
-	size_t nodecount = 0;
+	gsize nodecount = 0;
 	_vte_trie_printx(trie, "", &nodecount);
 	printf("Trie has %ld nodes.\n", (long) nodecount);
 }
@@ -843,13 +840,13 @@ dump_array(GValueArray *array)
 }
 
 static void
-convert_mbstowcs(const char *i, size_t ilen,
-		 gunichar *o, size_t *olen, size_t max_olen)
+convert_mbstowcs(const char *i, gsize ilen,
+		 gunichar *o, gsize *olen, gsize max_olen)
 {
 	VteConv conv;
-	size_t outlen;
+	gsize outlen;
 	conv = _vte_conv_open(VTE_CONV_GUNICHAR_TYPE, "UTF-8");
-	g_assert(conv != ((VteConv) -1));
+	g_assert(conv != VTE_INVALID_CONV);
 
 	memset(o, 0, max_olen);
 	outlen = max_olen;
@@ -867,7 +864,7 @@ main(int argc, char **argv)
 	GQuark quark;
 	gunichar buf[LINE_MAX];
 	const gunichar *consumed;
-	size_t buflen;
+	gsize buflen;
 
 	_vte_debug_parse_string(getenv("VTE_DEBUG_FLAGS"));
 
@@ -907,10 +904,10 @@ main(int argc, char **argv)
 	printf("`%s' = `%s'\n", "abc",
 	       _vte_trie_match(trie, buf, buflen,
 			       NULL, &consumed, &quark, &array));
-	printf("=> `%s' (%d)\n", g_quark_to_string(quark), consumed - buf);
+	printf("=> `%s' (%d)\n", g_quark_to_string(quark), (int)(consumed - buf));
 	if (array != NULL) {
 		dump_array(array);
-		_vte_matcher_free_params_array(array);
+		_vte_matcher_free_params_array(NULL, array);
 		array = NULL;
 	}
 
@@ -919,10 +916,10 @@ main(int argc, char **argv)
 	printf("`%s' = `%s'\n", "abcdef",
 	       _vte_trie_match(trie, buf, buflen,
 			       NULL, &consumed, &quark, &array));
-	printf("=> `%s' (%d)\n", g_quark_to_string(quark), consumed - buf);
+	printf("=> `%s' (%d)\n", g_quark_to_string(quark), (int)(consumed - buf));
 	if (array != NULL) {
 		dump_array(array);
-		_vte_matcher_free_params_array(array);
+		_vte_matcher_free_params_array(NULL, array);
 		array = NULL;
 	}
 
@@ -931,10 +928,10 @@ main(int argc, char **argv)
 	printf("`%s' = `%s'\n", "abcde",
 	       _vte_trie_match(trie, buf, buflen,
 			       NULL, &consumed, &quark, &array));
-	printf("=> `%s' (%d)\n", g_quark_to_string(quark), consumed - buf);
+	printf("=> `%s' (%d)\n", g_quark_to_string(quark), (int)(consumed - buf));
 	if (array != NULL) {
 		dump_array(array);
-		_vte_matcher_free_params_array(array);
+		_vte_matcher_free_params_array(NULL, array);
 		array = NULL;
 	}
 
@@ -943,10 +940,10 @@ main(int argc, char **argv)
 	printf("`%s' = `%s'\n", "abcdeg",
 	       _vte_trie_match(trie, buf, buflen,
 			       NULL, &consumed, &quark, &array));
-	printf("=> `%s' (%d)\n", g_quark_to_string(quark), consumed - buf);
+	printf("=> `%s' (%d)\n", g_quark_to_string(quark), (int)(consumed - buf));
 	if (array != NULL) {
 		dump_array(array);
-		_vte_matcher_free_params_array(array);
+		_vte_matcher_free_params_array(NULL, array);
 		array = NULL;
 	}
 
@@ -955,10 +952,10 @@ main(int argc, char **argv)
 	printf("`%s' = `%s'\n", "abc%deg",
 	       _vte_trie_match(trie, buf, buflen,
 			       NULL, &consumed, &quark, &array));
-	printf("=> `%s' (%d)\n", g_quark_to_string(quark), consumed - buf);
+	printf("=> `%s' (%d)\n", g_quark_to_string(quark), (int)(consumed - buf));
 	if (array != NULL) {
 		dump_array(array);
-		_vte_matcher_free_params_array(array);
+		_vte_matcher_free_params_array(NULL, array);
 		array = NULL;
 	}
 
@@ -967,10 +964,10 @@ main(int argc, char **argv)
 	printf("`%s' = `%s'\n", "abc10eg",
 	       _vte_trie_match(trie, buf, buflen,
 			       NULL, &consumed, &quark, &array));
-	printf("=> `%s' (%d)\n", g_quark_to_string(quark), consumed - buf);
+	printf("=> `%s' (%d)\n", g_quark_to_string(quark), (int)(consumed - buf));
 	if (array != NULL) {
 		dump_array(array);
-		_vte_matcher_free_params_array(array);
+		_vte_matcher_free_params_array(NULL, array);
 		array = NULL;
 	}
 
@@ -979,10 +976,10 @@ main(int argc, char **argv)
 	printf("`%s' = `%s'\n", "abc%eg",
 	       _vte_trie_match(trie, buf, buflen,
 			       NULL, &consumed, &quark, &array));
-	printf("=> `%s' (%d)\n", g_quark_to_string(quark), consumed - buf);
+	printf("=> `%s' (%d)\n", g_quark_to_string(quark), (int)(consumed - buf));
 	if (array != NULL) {
 		dump_array(array);
-		_vte_matcher_free_params_array(array);
+		_vte_matcher_free_params_array(NULL, array);
 		array = NULL;
 	}
 
@@ -991,10 +988,10 @@ main(int argc, char **argv)
 	printf("`%s' = `%s'\n", "abc%10eg",
 	       _vte_trie_match(trie, buf, buflen,
 			       NULL, &consumed, &quark, &array));
-	printf("=> `%s' (%d)\n", g_quark_to_string(quark), consumed - buf);
+	printf("=> `%s' (%d)\n", g_quark_to_string(quark), (int)(consumed - buf));
 	if (array != NULL) {
 		dump_array(array);
-		_vte_matcher_free_params_array(array);
+		_vte_matcher_free_params_array(NULL, array);
 		array = NULL;
 	}
 
@@ -1003,10 +1000,10 @@ main(int argc, char **argv)
 	printf("`%s' = `%s'\n", "abcBeg",
 	       _vte_trie_match(trie, buf, buflen,
 			       NULL, &consumed, &quark, &array));
-	printf("=> `%s' (%d)\n", g_quark_to_string(quark), consumed - buf);
+	printf("=> `%s' (%d)\n", g_quark_to_string(quark), (int)(consumed - buf));
 	if (array != NULL) {
 		dump_array(array);
-		_vte_matcher_free_params_array(array);
+		_vte_matcher_free_params_array(NULL, array);
 		array = NULL;
 	}
 
@@ -1015,10 +1012,10 @@ main(int argc, char **argv)
 	printf("`%s' = `%s'\n", "<esc>[25;26H",
 	       _vte_trie_match(trie, buf, buflen,
 			       NULL, &consumed, &quark, &array));
-	printf("=> `%s' (%d)\n", g_quark_to_string(quark), consumed - buf);
+	printf("=> `%s' (%d)\n", g_quark_to_string(quark), (int)(consumed - buf));
 	if (array != NULL) {
 		dump_array(array);
-		_vte_matcher_free_params_array(array);
+		_vte_matcher_free_params_array(NULL, array);
 		array = NULL;
 	}
 
@@ -1027,10 +1024,10 @@ main(int argc, char **argv)
 	printf("`%s' = `%s'\n", "<esc>[25;2",
 	       _vte_trie_match(trie, buf, buflen,
 			       NULL, &consumed, &quark, &array));
-	printf("=> `%s' (%d)\n", g_quark_to_string(quark), consumed - buf);
+	printf("=> `%s' (%d)\n", g_quark_to_string(quark), (int)(consumed - buf));
 	if (array != NULL) {
 		dump_array(array);
-		_vte_matcher_free_params_array(array);
+		_vte_matcher_free_params_array(NULL, array);
 	}
 
 	quark = 0;
@@ -1038,10 +1035,10 @@ main(int argc, char **argv)
 	printf("`%s' = `%s'\n", "<esc>[25L",
 	       _vte_trie_match(trie, buf, buflen,
 			       NULL, &consumed, &quark, &array));
-	printf("=> `%s' (%d)\n", g_quark_to_string(quark), consumed - buf);
+	printf("=> `%s' (%d)\n", g_quark_to_string(quark), (int)(consumed - buf));
 	if (array != NULL) {
 		dump_array(array);
-		_vte_matcher_free_params_array(array);
+		_vte_matcher_free_params_array(NULL, array);
 	}
 
 	quark = 0;
@@ -1049,10 +1046,10 @@ main(int argc, char **argv)
 	printf("`%s' = `%s'\n", "<esc>[25L<esc>[24L",
 	       _vte_trie_match(trie, buf, buflen,
 			       NULL, &consumed, &quark, &array));
-	printf("=> `%s' (%d)\n", g_quark_to_string(quark), consumed - buf);
+	printf("=> `%s' (%d)\n", g_quark_to_string(quark), (int)(consumed - buf));
 	if (array != NULL) {
 		dump_array(array);
-		_vte_matcher_free_params_array(array);
+		_vte_matcher_free_params_array(NULL, array);
 	}
 
 	quark = 0;
@@ -1060,10 +1057,10 @@ main(int argc, char **argv)
 	printf("`%s' = `%s'\n", "<esc>[25;26L",
 	       _vte_trie_match(trie, buf, buflen,
 			       NULL, &consumed, &quark, &array));
-	printf("=> `%s' (%d)\n", g_quark_to_string(quark), consumed - buf);
+	printf("=> `%s' (%d)\n", g_quark_to_string(quark), (int)(consumed - buf));
 	if (array != NULL) {
 		dump_array(array);
-		_vte_matcher_free_params_array(array);
+		_vte_matcher_free_params_array(NULL, array);
 	}
 
 	quark = 0;
@@ -1071,10 +1068,10 @@ main(int argc, char **argv)
 	printf("`%s' = `%s'\n", "<esc>]2;WoofWoofh",
 	       _vte_trie_match(trie, buf, buflen,
 			       NULL, &consumed, &quark, &array));
-	printf("=> `%s' (%d)\n", g_quark_to_string(quark), consumed - buf);
+	printf("=> `%s' (%d)\n", g_quark_to_string(quark), (int)(consumed - buf));
 	if (array != NULL) {
 		dump_array(array);
-		_vte_matcher_free_params_array(array);
+		_vte_matcher_free_params_array(NULL, array);
 		array = NULL;
 	}
 
@@ -1084,10 +1081,10 @@ main(int argc, char **argv)
 	printf("`%s' = `%s'\n", "<esc>]2;WoofWoofh<esc>]2;WoofWoofh",
 	       _vte_trie_match(trie, buf, buflen,
 			       NULL, &consumed, &quark, &array));
-	printf("=> `%s' (%d)\n", g_quark_to_string(quark), consumed - buf);
+	printf("=> `%s' (%d)\n", g_quark_to_string(quark), (int)(consumed - buf));
 	if (array != NULL) {
 		dump_array(array);
-		_vte_matcher_free_params_array(array);
+		_vte_matcher_free_params_array(NULL, array);
 		array = NULL;
 	}
 
@@ -1096,10 +1093,10 @@ main(int argc, char **argv)
 	printf("`%s' = `%s'\n", "<esc>]2;WoofWoofhfoo",
 	       _vte_trie_match(trie, buf, buflen,
 			       NULL, &consumed, &quark, &array));
-	printf("=> `%s' (%d)\n", g_quark_to_string(quark), consumed - buf);
+	printf("=> `%s' (%d)\n", g_quark_to_string(quark), (int)(consumed - buf));
 	if (array != NULL) {
 		dump_array(array);
-		_vte_matcher_free_params_array(array);
+		_vte_matcher_free_params_array(NULL, array);
 		array = NULL;
 	}
 
@@ -1108,3 +1105,11 @@ main(int argc, char **argv)
 	return 0;
 }
 #endif
+
+const struct _vte_matcher_class _vte_matcher_trie = {
+	(_vte_matcher_create_func)_vte_trie_new,
+	(_vte_matcher_add_func)_vte_trie_add,
+	(_vte_matcher_print_func)_vte_trie_print,
+	(_vte_matcher_match_func)_vte_trie_match,
+	(_vte_matcher_destroy_func)_vte_trie_free
+};
