@@ -76,6 +76,7 @@ struct _vte_xft_data {
 	gint scrollx, scrolly;
 	GPtrArray *locked_fonts[2];
 	guint cur_locked_fonts;
+	gboolean has_clip_mask;
 };
 
 /* protected by gdk_mutex */
@@ -261,7 +262,9 @@ _vte_xft_open_font_for_char (struct _vte_xft_font *font, gunichar c, GPtrArray *
 	/* No match? */
 	_vte_tree_insert (font->fontmap,
 			p, GINT_TO_POINTER (-FONT_INDEX_FUDGE));
-	g_warning (_ ("Can not find appropiate font for character U+%04x.\n"), c);
+	_vte_debug_print (VTE_DEBUG_MISC,
+			"Can not find appropiate font for character U+%04x.\n",
+			c);
 	return NULL;
 }
 static inline XftFont *
@@ -427,6 +430,7 @@ _vte_xft_start (struct _vte_draw *draw)
 		data->colormap = gdk_x11_colormap_get_xcolormap (gcolormap);
 		data->draw = XftDrawCreate (data->display, data->drawable,
 				data->visual, data->colormap);
+		data->has_clip_mask = FALSE;
 	}
 	g_assert (data->display == data->font->display);
 
@@ -513,18 +517,30 @@ _vte_xft_clip (struct _vte_draw *draw,
 	gint i, n;
 
 	gdk_region_get_rectangles (region, &rect, &n);
-	xrect = n > (gint) G_N_ELEMENTS (stack_rect) ?
-		g_new (XRectangle, n) :
-		stack_rect;
-	for (i = 0; i < n; i++) {
-		xrect[i].x = rect[i].x - data->x_offs;
-		xrect[i].y = rect[i].y - data->y_offs;
-		xrect[i].width = rect[i].width;
-		xrect[i].height = rect[i].height;
+	/* only enable clipping if we have to */
+	if (n > 1 ||
+			rect[0].width < draw->widget->allocation.width ||
+			rect[0].height < draw->widget->allocation.height) {
+		xrect = n > (gint) G_N_ELEMENTS (stack_rect) ?
+			g_new (XRectangle, n) :
+			stack_rect;
+		for (i = 0; i < n; i++) {
+			xrect[i].x = rect[i].x - data->x_offs;
+			xrect[i].y = rect[i].y - data->y_offs;
+			xrect[i].width = rect[i].width;
+			xrect[i].height = rect[i].height;
+		}
+		XftDrawSetClipRectangles (data->draw, 0, 0, xrect, n);
+		data->has_clip_mask = TRUE;
+		if (xrect != stack_rect)
+			g_free (xrect);
+	} else {
+		if (data->has_clip_mask) {
+			XftDrawSetClip (data->draw, NULL);
+			data->has_clip_mask = FALSE;
+		}
 	}
-	XftDrawSetClipRectangles (data->draw, 0, 0, xrect, n);
-	if (xrect != stack_rect)
-		g_free (xrect);
+
 	g_free (rect);
 }
 
